searchData={"content_type":"text/markdown","items":[{"doc":"Bandit is an HTTP server for Plug and WebSock apps.\n\nAs an HTTP server, Bandit's primary goal is to act as 'glue' between client connections managed\nby [Thousand Island](https://github.com/mtrudel/thousand_island) and application code defined\nvia the [Plug](https://github.com/elixir-plug/plug) and/or\n[WebSock](https://github.com/phoenixframework/websock) APIs. As such there really isn't a whole lot of\nuser-visible surface area to Bandit, and as a consequence the API documentation presented here\nis somewhat sparse. This is by design! Bandit is intended to 'just work' in almost all cases;\nthe only thought users typically have to put into Bandit comes in the choice of which options (if\nany) they would like to change when starting a Bandit server. The sparseness of the Bandit API\nshould not be taken as an indicator of the comprehensiveness or robustness of the project.","ref":"Bandit.html","title":"Bandit","type":"module"},{"doc":"Bandit fully supports Phoenix. Phoenix applications which use WebSockets for\nfeatures such as Channels or LiveView require Phoenix 1.7 or later.\n\nUsing Bandit to host your Phoenix application couldn't be simpler:\n\n1. Add Bandit as a dependency in your Phoenix application's `mix.exs`:\n\n    ```elixir\n    {:bandit, \"~> 1.0-pre\"}\n    ```\n2. Add the following `adapter:` line to your endpoint configuration in `config/config.exs`, as in the following example:\n\n     ```elixir\n     # config/config.exs\n\n     config :your_app, YourAppWeb.Endpoint,\n       adapter: Bandit.PhoenixAdapter, # <---- ADD THIS LINE\n       url: [host: \"localhost\"],\n       render_errors: ...\n     ```\n3. That's it! **You should now see messages at startup indicating that Phoenix is\n   using Bandit to serve your endpoint**, and everything should 'just work'. Note\n   that if you have set any exotic configuration options within your endpoint,\n   you may need to update that configuration to work with Bandit; see the\n   [Bandit.PhoenixAdapter](https://hexdocs.pm/bandit/Bandit.PhoenixAdapter.html)\n   documentation for more information.","ref":"Bandit.html#module-using-bandit-with-phoenix","title":"Using Bandit With Phoenix - Bandit","type":"module"},{"doc":"Using Bandit to host your own Plug is very straightforward. Assuming you have\na Plug module implemented already, you can host it within Bandit by adding\nsomething similar to the following to your application's `Application.start/2`\nfunction:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    {Bandit, plug: MyApp.MyPlug}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\nFor less formal usage, you can also start Bandit using the same configuration\noptions via the `Bandit.start_link/1` function:\n\n```elixir\n# Start an http server on the default port 4000, serving MyApp.MyPlug\nBandit.start_link(plug: MyPlug)\n```","ref":"Bandit.html#module-using-bandit-with-plug-applications","title":"Using Bandit With Plug Applications - Bandit","type":"module"},{"doc":"A number of options are defined when starting a server. The complete list is\ndefined by the [`t:Bandit.options/0`](https://hexdocs.pm/bandit/Bandit.html#summary) type.","ref":"Bandit.html#module-configuration","title":"Configuration - Bandit","type":"module"},{"doc":"By far the most common stumbling block encountered when setting up an HTTPS\nserver involves configuring key and certificate data. Bandit is comparatively\neasy to set up in this regard, with a working example looking similar to the\nfollowing:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    {Bandit,\n     plug: MyPlug,\n     scheme: :https,\n     certfile: \"/absolute/path/to/cert.pem\",\n     keyfile: \"/absolute/path/to/key.pem\"}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"Bandit.html#module-setting-up-an-https-server","title":"Setting up an HTTPS Server - Bandit","type":"module"},{"doc":"If you're using Bandit to run a Phoenix application as suggested above, there is\nnothing more for you to do; WebSocket support will 'just work'.\n\nIf you wish to interact with WebSockets at a more fundamental level, the\n[WebSock](https://hexdocs.pm/websock/WebSock.html) and\n[WebSockAdapter](https://hexdocs.pm/websock_adapter/WebSockAdapter.html) libraries\nprovides a generic abstraction for WebSockets (very similar to how Plug is\na generic abstraction on top of HTTP). Bandit fully supports all aspects of\nthese libraries.","ref":"Bandit.html#module-websocket-support","title":"WebSocket Support - Bandit","type":"module"},{"doc":"Starts a Bandit server using the provided arguments. See `t:options/0` for specific options to\npass to this function.","ref":"Bandit.html#start_link/1","title":"Bandit.start_link/1","type":"function"},{"doc":"Options to configure the deflate library used for HTTP compression","ref":"Bandit.html#t:deflate_options/0","title":"Bandit.deflate_options/0","type":"type"},{"doc":"Options to configure the HTTP/1 stack in Bandit\n\n* `enabled`: Whether or not to serve HTTP/1 requests. Defaults to true\n* `max_request_line_length`: The maximum permitted length of the request line\n  (expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes\n* `max_header_length`: The maximum permitted length of any single header (combined\n  key & value, expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes\n* `max_header_count`: The maximum permitted number of headers in an HTTP/1.1 request.\n  Defaults to 50 headers\n* `max_requests`: The maximum number of requests to serve in a single\n  HTTP/1.1 connection before closing the connection. Defaults to 0 (no limit)\n* `compress`: Whether or not to attempt compression of responses via content-encoding\n  negotiation as described in\n  [RFC9110§8.4](https://www.rfc-editor.org/rfc/rfc9110.html#section-8.4). Defaults to true\n* `deflate_options`: A keyword list of options to set on the deflate library. A complete list can\n  be found at `t:deflate_options/0`","ref":"Bandit.html#t:http_1_options/0","title":"Bandit.http_1_options/0","type":"type"},{"doc":"Options to configure the HTTP/2 stack in Bandit\n\n* `enabled`: Whether or not to serve HTTP/2 requests. Defaults to true\n* `max_header_key_length`: The maximum permitted length of any single header key\n  (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes\n* `max_header_value_length`: The maximum permitted length of any single header value\n  (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes\n* `max_header_count`: The maximum permitted number of headers in an HTTP/2 request.\n  Defaults to 50 headers\n* `max_requests`: The maximum number of requests to serve in a single\n  HTTP/2 connection before closing the connection. Defaults to 0 (no limit)\n* `default_local_settings`: Options to override the default values for local HTTP/2\n  settings. Values provided here will override the defaults specified in RFC9113§6.5.2\n* `compress`: Whether or not to attempt compression of responses via content-encoding\n  negotiation as described in\n  [RFC9110§8.4](https://www.rfc-editor.org/rfc/rfc9110.html#section-8.4). Defaults to true\n* `deflate_options`: A keyword list of options to set on the deflate library. A complete list can\n  be found at `t:deflate_options/0`","ref":"Bandit.html#t:http_2_options/0","title":"Bandit.http_2_options/0","type":"type"},{"doc":"Possible top-level options to configure a Bandit server\n\n* `plug`: The Plug to use to handle connections. Can be specified as `MyPlug` or `{MyPlug, plug_opts}`\n* `scheme`: One of `:http` or `:https`. If `:https` is specified, you will also need to specify\n  valid `certfile` and `keyfile` values (or an equivalent value within\n  `thousand_island_options.transport_options`). Defaults to `:http`\n* `port`: The TCP port to listen on. This option is offered as a convenience and actually sets\n  the option of the same name within `thousand_island_options`. If ia string value is passed, it\n  will be parsed as an integer. Defaults to 4000 if `scheme` is `:http`, and 4040 if `scheme` is\n  `:https`\n* `ip`:  The interface(s) to listen on. This option is offered as a convenience and actually sets the\n  option of the same name within `thousand_island_options.transport_options`. Can be specified as:\n    * `{1, 2, 3, 4}` for IPv4 addresses\n    * `{1, 2, 3, 4, 5, 6, 7, 8}` for IPv6 addresses\n    * `:loopback` for local loopback (ie: `127.0.0.1`)\n    * `:any` for all interfaces (ie: `0.0.0.0`)\n    * `{:local, \"/path/to/socket\"}` for a Unix domain socket. If this option is used, the `port`\n      option *must* be set to `0`\n* `keyfile`: The path to a file containing the SSL key to use for this server. This option is\n  offered as a convenience and actually sets the option of the same name within\n  `thousand_island_options.transport_options`. If a relative path is used here, you will also\n  need to set the `otp_app` parameter and ensure that the named file is part of your application\n  build\n* `certfile`: The path to a file containing the SSL certificate to use for this server. This option is\n  offered as a convenience and actually sets the option of the same name within\n  `thousand_island_options.transport_options`. If a relative path is used here, you will also\n  need to set the `otp_app` parameter and ensure that the named file is part of your application\n  build\n* `otp_app`: Provided as a convenience when using relative paths for `keyfile` and `certfile`\n* `cipher_suite`: Used to define a pre-selected set of ciphers, as described by\n  `Plug.SSL.configure/1`. Optional, can be either `:strong` or `:compatible`\n* `display_plug`: The plug to use when describing the connection in logs. Useful for situations\n  such as Phoenix code reloading where you have a 'wrapper' plug but wish to refer to the\n  connection by the endpoint name\n* `startup_log`: The log level at which Bandit should log startup info.\n  Defaults to `:info` log level, can be set to false to disable it\n* `thousand_island_options`: A list of options to pass to Thousand Island. Bandit sets some\n  default values in this list based on your top-level configuration; these values will be\n  overridden by values appearing here. A complete list can be found at\n  `t:ThousandIsland.options/0`\n* `http_1_options`: A list of options to configure Bandit's HTTP/1 stack. A complete list can\n  be found at `t:http_1_options/0`\n* `http_2_options`: A list of options to configure Bandit's HTTP/2 stack. A complete list can\n  be found at `t:http_2_options/0`\n* `websocket_options`: A list of options to configure Bandit's WebSocket stack. A complete list can\n  be found at `t:websocket_options/0`","ref":"Bandit.html#t:options/0","title":"Bandit.options/0","type":"type"},{"doc":"Options to configure the WebSocket stack in Bandit\n\n* `enabled`: Whether or not to serve WebSocket upgrade requests. Defaults to true\n* `max_frame_size`: The maximum size of a single WebSocket frame (expressed as\n  a number of bytes on the wire). Defaults to 0 (no limit)\n* `validate_text_frames`: Whether or not to validate text frames as being UTF-8. Strictly\n  speaking this is required per RFC6455§5.6, however it can be an expensive operation and one\n  that may be safely skipped in some situations. Defaults to true\n* `compress`: Whether or not to allow per-message deflate compression globally. Note that\n  upgrade requests still need to set the `compress: true` option in `connection_opts` on\n  a per-upgrade basis for compression to be negotiated (see 'WebSocket Support' section below\n  for details). Defaults to `true`","ref":"Bandit.html#t:websocket_options/0","title":"Bandit.websocket_options/0","type":"type"},{"doc":"Settings as defined in RFC9113§6.5.2","ref":"Bandit.HTTP2.Settings.html","title":"Bandit.HTTP2.Settings","type":"module"},{"doc":"A collection of settings as defined in RFC9113§6.5","ref":"Bandit.HTTP2.Settings.html#t:t/0","title":"Bandit.HTTP2.Settings.t/0","type":"type"},{"doc":"Logging conveniences for Bandit servers\n\nAllows dynamically adding and altering the log level used to trace connections\nwithin a Bandit server via the use of telemetry hooks. Should you wish\nto do your own logging or tracking of these events, a complete list of the\ntelemetry events emitted by Bandit is described in the module documentation\nfor `Bandit.Telemetry`.\n\nThe logging included in this module is concerned specifically with protocol level events.\nShould you wish to log lower level transport concens, there are similar functions to these in\nthe `ThousandIsland.Logger` module. Corresponding telemetry events are described in the\nmodule documentation for `ThousandIsland.Telemetry`.","ref":"Bandit.Logger.html","title":"Bandit.Logger","type":"module"},{"doc":"Start logging Bandit at the specified log level. Valid values for log\nlevel are `:error` and `:info`. Enabling a given log level implicitly enables all higher log\nlevels as well.","ref":"Bandit.Logger.html#attach_logger/1","title":"Bandit.Logger.attach_logger/1","type":"function"},{"doc":"Stop logging Thousand Island at the specified log level. Disabling a given log\nlevel implicitly disables all lower log levels as well.","ref":"Bandit.Logger.html#detach_logger/1","title":"Bandit.Logger.detach_logger/1","type":"function"},{"doc":"Supported log levels","ref":"Bandit.Logger.html#t:log_level/0","title":"Bandit.Logger.log_level/0","type":"type"},{"doc":"A Bandit adapter for Phoenix.\n\nThis adapter provides out-of-the-box support for all aspects of Phoenix 1.7 and later. Earlier\nversions of Phoenix will work with this adapter, but without support for WebSockets.\n\nTo use this adapter, your project will need to include Bandit as a dependency:\n\n```elixir\n{:bandit, \"~> 1.0-pre\"}\n```\n\nOnce Bandit is included as a dependency of your Phoenix project, add the following `adapter:`\nline to your endpoint configuration in `config/config.exs`, as in the following example:\n\n```\n# config/config.exs\n\nconfig :your_app, YourAppWeb.Endpoint,\n  adapter: Bandit.PhoenixAdapter, # <---- ADD THIS LINE\n  url: [host: \"localhost\"],\n  render_errors: ...\n```\n\nThat's it! **After restarting Phoenix you should see the startup message indicate that it is being\nserved by Bandit**, and everything should 'just work'. Note that if you have set any exotic\nconfiguration options within your endpoint, you may need to update that configuration to work\nwith Bandit; see below for details.","ref":"Bandit.PhoenixAdapter.html","title":"Bandit.PhoenixAdapter","type":"module"},{"doc":"This adapter supports the standard Phoenix structure for endpoint configuration. Top-level keys for\n`:http` and `:https` are supported, and configuration values within each of those are interpreted\nas raw Bandit configuration as specified by `t:Bandit.options/0`. Bandit's confguration supports\nall values used in a standard out-of-the-box Phoenix application, so if you haven't made any\nsubstantial changes to your endpoint configuration things should 'just work' for you.\n\nIn the event that you *have* made advanced changes to your endpoint configuration, you may need\nto update this config to work with Bandit. Consult Bandit's documentation at\n`t:Bandit.options/0` for details.","ref":"Bandit.PhoenixAdapter.html#module-endpoint-configuration","title":"Endpoint configuration - Bandit.PhoenixAdapter","type":"module"},{"doc":"The following telemetry spans are emitted by bandit\n\n## `[:bandit, :request, *]`\n\nRepresents Bandit handling a specific client HTTP request\n\nThis span is started by the following event:\n\n* `[:bandit, :request, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n\nThis span is ended by the following event:\n\n* `[:bandit, :request, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `req_header_end_time`: The time that header reading completed, in `:native` units\n    * `req_body_start_time`: The time that request body reading started, in `:native` units.\n    * `req_body_end_time`: The time that request body reading completed, in `:native` units\n    * `req_line_bytes`: The length of the request line, in octets. Includes all line breaks.\n      Not included for HTTP/2 requests\n    * `req_header_bytes`: The length of the request headers, in octets. Includes all line\n      breaks. Not included for HTTP/2 requests\n    * `req_body_bytes`: The length of the request body, in octets\n    * `resp_start_time`: The time that the response started, in `:native` units\n    * `resp_end_time`: The time that the response completed, in `:native` units. Not included\n      for chunked responses\n    * `resp_line_bytes`: The length of the reponse line, in octets. Includes all line breaks.\n      Not included for HTTP/2 requests\n    * `resp_header_bytes`: The length of the reponse headers, in octets. Includes all line\n      breaks. Not included for HTTP/2 requests\n    * `resp_body_bytes`: The length of the reponse body, in octets. If the response is\n      compressed, this is the size of the compressed payload as sent on the wire. Set to 0 for\n      chunked responses\n    * `resp_uncompressed_body_bytes`: The length of the original, uncompressed body. Only\n      included for responses which are compressed\n    * `resp_compression_method`: The method of compression, as sent in the `Content-Encoding`\n      header of the response. Only included for responses which are compressed\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `method`: The HTTP method of the request. If Bandit could not determine the method, this\n      will be nil\n    * `request_target`: The raw request target for the request. Returned as a tuple comprising the\n      constituent parts of the request target: `{scheme, host, port, path}`. If Bandit could not\n      determine the request target, this will be nil\n    * `status`: The numeric response status code of the response\n    * `conn`: The `Plug.Conn` representing this connection. Not present in cases where `error`\n      is also set\n    * `stream_id`: The stream ID of the request, if part of an HTTP/2 connection\n    * `error`: The error that caused the span to end, if it ended in error\n\nThe following events may be emitted within this span:\n\n* `[:bandit, :request, :exception]`\n\n    The request for this span ended unexpectedly\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `kind`: The kind of unexpected condition, typically `:exit`\n    * `exception`: The exception which caused this unexpected termination\n    * `stacktrace`: The stacktrace of the location which caused this unexpected termination\n\n## `[:bandit, :websocket, *]`\n\nRepresents Bandit handling a WebSocket connection\n\nThis span is started by the following event:\n\n* `[:bandit, :websocket, :start]`\n\n    Represents the start of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `compress`: Details about the compression configuration for this connection\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `origin_telemetry_span_context`: The span context of the Bandit `:request` span from which\n      this connection originated\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n\nThis span is ended by the following event:\n\n* `[:bandit, :websocket, :stop]`\n\n    Represents the end of the span\n\n    This event contains the following measurements:\n\n    * `monotonic_time`: The time of this event, in `:native` units\n    * `duration`: The span duration, in `:native` units\n    * `recv_text_frame_count`: The number of text frames received\n    * `recv_text_frame_bytes`: The total number of bytes received in the payload of text frames\n    * `recv_binary_frame_count`: The number of binary frames received\n    * `recv_binary_frame_bytes`: The total number of bytes received in the payload of binary frames\n    * `recv_ping_frame_count`: The number of ping frames received\n    * `recv_ping_frame_bytes`: The total number of bytes received in the payload of ping frames\n    * `recv_pong_frame_count`: The number of pong frames received\n    * `recv_pong_frame_bytes`: The total number of bytes received in the payload of pong frames\n    * `recv_connection_close_frame_count`: The number of connection close frames received\n    * `recv_connection_close_frame_bytes`: The total number of bytes received in the payload of connection close frames\n    * `recv_continuation_frame_count`: The number of continuation frames received\n    * `recv_continuation_frame_bytes`: The total number of bytes received in the payload of continuation frames\n    * `send_text_frame_count`: The number of text frames sent\n    * `send_text_frame_bytes`: The total number of bytes sent in the payload of text frames\n    * `send_binary_frame_count`: The number of binary frames sent\n    * `send_binary_frame_bytes`: The total number of bytes sent in the payload of binary frames\n    * `send_ping_frame_count`: The number of ping frames sent\n    * `send_ping_frame_bytes`: The total number of bytes sent in the payload of ping frames\n    * `send_pong_frame_count`: The number of pong frames sent\n    * `send_pong_frame_bytes`: The total number of bytes sent in the payload of pong frames\n    * `send_connection_close_frame_count`: The number of connection close frames sent\n    * `send_connection_close_frame_bytes`: The total number of bytes sent in the payload of connection close frames\n    * `send_continuation_frame_count`: The number of continuation frames sent\n    * `send_continuation_frame_bytes`: The total number of bytes sent in the payload of continuation frames\n\n    This event contains the following metadata:\n\n    * `telemetry_span_context`: A unique identifier for this span\n    * `origin_telemetry_span_context`: The span context of the Bandit `:request` span from which\n      this connection originated\n    * `connection_telemetry_span_context`: The span context of the Thousand Island `:connection`\n      span which contains this request\n    * `error`: The error that caused the span to end, if it ended in error","ref":"Bandit.Telemetry.html","title":"Bandit.Telemetry","type":"module"},{"doc":"","ref":"Bandit.Telemetry.html#monotonic_time/0","title":"Bandit.Telemetry.monotonic_time/0","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#span_exception/4","title":"Bandit.Telemetry.span_exception/4","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#t:t/0","title":"Bandit.Telemetry.t/0","type":"opaque"},{"doc":"Raised by Bandit adapters if a body is attempted to be read more than once per request","ref":"Bandit.BodyAlreadyReadError.html","title":"Bandit.BodyAlreadyReadError","type":"exception"},{"doc":"","ref":"Bandit.HTTP2.Stream.StreamError.html","title":"Bandit.HTTP2.Stream.StreamError","type":"exception"},{"doc":"# Changelog for 1.0.0-pre","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-12-12-aug-2023","title":"1.0.0-pre.12 (12 Aug 2023) - Changelog","type":"extras"},{"doc":"* Bump ThousandIsland to 1.0.0-pre.7 to fix leaking file descriptors on\n  `Plug.Conn.sendfile/5` calls (thanks @Hermanverschooten!)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-11-11-aug-2023","title":"1.0.0-pre.11 (11 Aug 2023) - Changelog","type":"extras"},{"doc":"* **BREAKING CHANGE** Move `conn` value in telemetry events from measurements to metadata","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Add `method`, `request_target` and `status` fields to telemetry metadata on HTTP stop events\n* Improve RFC compliance regarding cache-related headers on deflated responses (#207, thanks @tanguilp!)\n* Bump to Thousand Island `1.0.0-pre.6`\n* Doc improvements (particularly around implementation notes)\n* Typespec improvements (thanks @moogle19!)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-10-28-jun-2023","title":"1.0.0-pre.10 (28 Jun 2023) - Changelog","type":"extras"},{"doc":"* Add support for `Plug.Conn.inform/3` on HTTP/1 connections (#180)\n* Add support for h2c upgrades (#186, thanks @alisinabh!)\n* Internal refactoring of HTTP/1 content-length encoded body reads (#184, #190,\n  thanks @asakura & @moogle19!)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Bump Thousand Island to 1.0.0-pre.6 (gaining support for suspend/resume API)\n* Drop Elixir 1.12 as a supported target (it should continue to work, but is no\n  longer covered by CI)","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Fix crash when Plug used `Plug.Conn.get_peer_data/1` function on HTTP/1\n  connections (#170, thanks @moogle19!)\n* Fix port behaviour when connecting over unix socket (#176, thanks @asakura\n  & @ibarchenkov!)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-9-16-jun-2023","title":"1.0.0-pre.9 (16 Jun 2023) - Changelog","type":"extras"},{"doc":"* Use new ThousandIsland APIs for socket info (#167, thanks @asakura!)","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Handle nil connection close reason when closing a WebSocket","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-8-15-jun-2023","title":"1.0.0-pre.8 (15 Jun 2023) - Changelog","type":"extras"},{"doc":"* Further improve logging on WebSocket upgrade errors (#149)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-7-14-jun-2023","title":"1.0.0-pre.7 (14 Jun 2023) - Changelog","type":"extras"},{"doc":"* Refactor HTTP/1 read routines (#158 & #166, thanks @asakura!)\n* Improve logging on WebSocket upgrade errors (#149)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Override any content-length headers that may have been set by Plug (#165)\n* Send content-length on HTTP/2 responses where appropriate (#165)","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Send correct content-length header when sending deflated response (#151)\n* Do not attempt to deflate if Plug sends a content-encoding header (#165)\n* Improve corner case handling of content-length request header (#163, thanks\n  @ryanwinchester!)\n* Handle case where ThousandIsland returns error tuples on some helper routines\n  (#162)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-pre-6-8-jun-2023","title":"1.0.0-pre.6 (8 Jun 2023) - Changelog","type":"extras"},{"doc":"* Always use the declaed scheme if declared in a request-line or `:scheme`\n  pseudo-header (#159)\n* Internal tidying (thanks @asakura!)","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-pre-5-2-jun-2023","title":"1.0.0-pre.5 (2 Jun 2023) - Changelog","type":"extras"},{"doc":"* Total overhaul of typespecs throughout the library (thanks @asakura!)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-pre-4-23-may-2023","title":"1.0.0-pre.4 (23 May 2023) - Changelog","type":"extras"},{"doc":"* Performance / correctness improvements to header length validation (#143,\n  thanks @moogle19!)\n* Performance improvements to host header port parsing (#145 & #147, thanks\n  @ryanwinchester!)\n* Improve WebSocket upgrade failure error messages to aid in diagnosis (#152)\n\n#","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Consolidate credo config (#146, thanks @ryanwinchester!)\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Fix error in suggested version dependencies during 1.0-pre series (#142,\n  thanks @cvkmohan!)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-pre-3-3-may-2023","title":"1.0.0-pre.3 (3 May 2023) - Changelog","type":"extras"},{"doc":"* Respect read timeout for HTTP/1 keepalives (#140)\n* Support Websock 0.5.1, including support for optional `c:WebSock.terminate/2`\n  (#131)\n\n#","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Use Req instead of Finch in tests (#137)\n* Improve a few corner cases in tests (#136)","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-pre-2-24-apr-2023","title":"1.0.0-pre.2 (24 Apr 2023) - Changelog","type":"extras"},{"doc":"* Don't require transport_options to be a keyword list (#130, thanks @justinludwig!)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#1-0-0-pre-1-21-apr-2023","title":"1.0.0-pre.1 (21 Apr 2023) - Changelog","type":"extras"},{"doc":"* Update Thousand Island dependency to 1.0-pre\n\n# Changelog for 0.7.x","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-7-11-apr-2023","title":"0.7.7 (11 Apr 2023) - Changelog","type":"extras"},{"doc":"* Bandit will now raise an error at startup if no plug is specified in config\n  (thanks @moogle19!)\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Fix crash at startup when using `otp_app` option (thanks @moogle19!)\n* Minor doc formatting fixes","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-6-9-apr-2023","title":"0.7.6 (9 Apr 2023) - Changelog","type":"extras"},{"doc":"* **BREAKING CHANGE** Rename top-level `options` field to `thousand_island_options`\n* **BREAKING CHANGE** Rename `deflate_opts` to `deflate_options` where used\n* Massive overhaul of documentation to use types where possible\n* Bandit now uses a term of the form `{Bandit, ref()}` for `id` in our child spec\n* Bumped to Thousand Island 0.6.7. `num_connections` is now 16384 by default\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Added top level support for the following convenience parameters:\n  * `port` can now be set at the top level of your configuration\n  * `ip` can now be set at the top level of your configuration\n  * `keyfile` and `certfile` can now be set at the top level of your configuration\n* Transport options are now validated by `Plug.SSL.configure/1` when starting\n  an HTTPS server\n* Rely on Thousand Island to validate options specified in `thousand_island_options`. This should avoid cases like #125 in the future.","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-5-4-apr-2023","title":"0.7.5 (4 Apr 2023) - Changelog","type":"extras"},{"doc":"* Drop explicit support for Elixir 1.11 since we no longer test it in CI (should\n  still work, just that it's now at-your-own-risk)\n* Add logo to ex_doc and README\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Allow access to Thousand Island's underlying `shutdown_timeout` option\n* Fix test errors that cropped up in OTP 26","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-4-27-mar-2023","title":"0.7.4 (27 Mar 2023) - Changelog","type":"extras"},{"doc":"* Calling `Plug.Conn` adapter functions for HTTP/2 based requests are no longer\n  restricted to being called from the process which called `c:Plug.call/2`\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Added `startup_log` to control whether / how Bandit logs the bound host & port\n  at startup (Thanks @danschultzer)\n* Improved logging when the configured port is in use at startup (Thanks\n  @danschultzer)\n* Update to Thousand Island 0.6.5","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-3-20-mar-2023","title":"0.7.3 (20 Mar 2023) - Changelog","type":"extras"},{"doc":"* Added advanced `handler_module` configuration option to `options`\n\n#","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Support returning `x-gzip` as negotiated `content-encoding` (previously would\n  negotiate a request for `x-gzip` as `gzip`)","ref":"changelog.html#fixes","title":"Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-2-18-mar-2023","title":"0.7.2 (18 Mar 2023) - Changelog","type":"extras"},{"doc":"* Added HTTP compression via 'Content-Encoding' negotiation, enabled by default.\n  Configuration is available; see [Bandit\n  docs](https://hexdocs.pm/bandit/Bandit.html#module-config-options) for details\n\n#","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Minor refactor of internal HTTP/2 plumbing. No user visible changes","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-1-17-mar-2023","title":"0.7.1 (17 Mar 2023) - Changelog","type":"extras"},{"doc":"* Update documentation & messaging to refer to RFC911x RFCs where appropriate\n* Validate top-level config options at startup\n* Revise Phoenix adapter to support new config options\n* Doc updates","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-7-0-17-mar-2023","title":"0.7.0 (17 Mar 2023) - Changelog","type":"extras"},{"doc":"* Add configuration points for various parameters within the HTTP/1, HTTP/2 and\n  WebSocket stacks. See [Bandit\n  docs](https://hexdocs.pm/bandit/Bandit.html#module-config-options) for details\n\n# Changelog for 0.6.x","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-11-17-mar-2023","title":"0.6.11 (17 Mar 2023) - Changelog","type":"extras"},{"doc":"* Modified telemetry event payloads to match the conventions espoused by\n  `:telemetry.span/3`\n* Default shutdown timeout is now 15s (up from 5s)\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"* Update to Thosuand Island 0.6.4 (from 0.6.2)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-10-10-mar-2023","title":"0.6.10 (10 Mar 2023) - Changelog","type":"extras"},{"doc":"* Support explicit setting of WebSocket close codes & reasons as added in WebSock\n0.5.0","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-9-20-feb-2023","title":"0.6.9 (20 Feb 2023) - Changelog","type":"extras"},{"doc":"* Add comprehensive Telemetry support within Bandit, as documented in the\n  `Bandit.Telemetry` module\n* Update our ThousandIsland dependnecy to pull in Thousand Island's newly\n  updated Telemetry support as documented in the `ThousandIsland.Telemetry`\n  module\n* Fix parsing of host / request headers which contain IPv6 addresses (#97).\n  Thanks @derekkraan!\n\n# Changes\n\n* Use Plug's list of response code reason phrases (#96). Thanks @jclem!\n* Minor doc updates","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-8-31-jan-2023","title":"0.6.8 (31 Jan 2023) - Changelog","type":"extras"},{"doc":"* Close WebSocket connections with a code of 1000 (instead of 1001) when\n  shutting down the server (#89)\n* Use 100 acceptor processes by default (instead of 10)\n* Improvements to make WebSocket frame masking faster","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-7-17-jan-2023","title":"0.6.7 (17 Jan 2023) - Changelog","type":"extras"},{"doc":"* Remove logging entirely when client connections do not contain a valid protocol\n* Refactor WebSocket support for about a 20% performance lift\n\n#","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Add `nodelay` option to test suite to fix artificially slow WebSocket perf tests","ref":"changelog.html#bug-fixes","title":"Bug Fixes - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-6-11-jan-2023","title":"0.6.6 (11 Jan 2023) - Changelog","type":"extras"},{"doc":"* Log useful message when a TLS connection is made to plaintext server (#74)","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"#","ref":"changelog.html#0-6-5-10-jan-2023","title":"0.6.5 (10 Jan 2023) - Changelog","type":"extras"},{"doc":"* Update Thousand Island to 0.5.15 (quiets logging in timeout cases)\n* Quiet logging in when client connections do not contain a valid protocol\n* Refactor HTTP/1 for about a 20% performance lift\n* Add WebSocket support to CI benchmark workflow\n* Doc updates\n\n#","ref":"changelog.html#enhancements","title":"Enhancements - Changelog","type":"extras"},{"doc":"* Allow multiple instances of Bandit to be started in the same node (#75)\n* Improve error handling in HTTP/1 when protocol errors are encountered (#74)","ref":"changelog.html#bug-fixes","title":"Bug Fixes - Changelog","type":"extras"},{"doc":"![Bandit](https://github.com/mtrudel/bandit/raw/main/assets/readme_logo.png#gh-light-mode-only)\n![Bandit](https://github.com/mtrudel/bandit/raw/main/assets/readme_logo-darkmode.png#gh-dark-mode-only)\n\n[![Build Status](https://github.com/mtrudel/bandit/workflows/Elixir%20CI/badge.svg)](https://github.com/mtrudel/bandit/actions)\n[![Docs](https://img.shields.io/badge/api-docs-green.svg?style=flat)](https://hexdocs.pm/bandit)\n[![Hex.pm](https://img.shields.io/hexpm/v/bandit.svg?style=flat&color=blue)](https://hex.pm/packages/bandit)\n\nBandit is an HTTP server for Plug and WebSock apps.\n\nBandit is written entirely in Elixir and is built atop [Thousand\nIsland](https://github.com/mtrudel/thousand_island). It can serve HTTP/1.x,\nHTTP/2 and WebSocket clients over both HTTP and HTTPS. It is written with\ncorrectness, clarity & performance as fundamental goals.\n\nIn [ongoing automated performance\ntests](https://github.com/mtrudel/bandit/actions/runs/4449308920),\nBandit's HTTP/1.x engine is up to 4x faster than Cowboy depending on the number of concurrent\nrequests. When comparing HTTP/2 performance, Bandit is up to 1.5x faster than Cowboy. This is\npossible because Bandit has been built from the ground up for use with Plug applications; this\nfocus pays dividends in both performance and also in the approachability of the code base.\n\nBandit also emphasizes correctness. Its HTTP/2 implementation scores 100% on the\n[h2spec](https://github.com/summerwind/h2spec) suite in strict mode, and its\nWebSocket implementation scores 100% on the\n[Autobahn](https://github.com/crossbario/autobahn-testsuite) test suite, both of\nwhich run as part of Bandit's comprehensive CI suite. Extensive unit test,\ncredo, dialyzer, and performance regression test coverage round out a test suite\nthat ensures that Bandit is and will remain a platform you can count on.\n\nLastly, Bandit exists to demystify the lower layers of infrastructure code. In a world where\nThe New Thing is nearly always adding abstraction on top of abstraction, it's important to have\nfoundational work that is approachable & understandable by users above it in the stack.","ref":"readme.html","title":"README","type":"extras"},{"doc":"* Implement comprehensive support for HTTP/1.0 through HTTP/2 & WebSockets (and\n  beyond) backed by obsessive RFC literacy and automated conformance testing\n* Aim for minimal internal policy and HTTP-level configuration. Delegate to Plug & WebSock as much as\n  possible, and only interpret requests to the extent necessary to safely manage a connection\n  & fulfill the requirements of safely supporting protocol correctness\n* Prioritize (in order): correctness, clarity, performance. Seek to remove the mystery of\n  infrastructure code by being approachable and easy to understand\n* Along with our companion library [Thousand\n  Island](https://github.com/mtrudel/thousand_island), become the go-to HTTP\n  & low-level networking stack of choice for the Elixir community by being\n  reliable, efficient, and approachable","ref":"readme.html#project-goals","title":"Project Goals - README","type":"extras"},{"doc":"* Complete support for [Phoenix](https://github.com/phoenixframework/phoenix) applications (WebSocket\n  support requires Phoenix 1.7+)\n* Complete support of the [Plug API](https://github.com/elixir-plug/plug)\n* Complete support of the [WebSock API](https://github.com/phoenixframework/websock)\n* Complete server support for HTTP/1.x as defined in [RFC\n  9112](https://datatracker.ietf.org/doc/html/rfc9112) & [RFC\n  9110](https://datatracker.ietf.org/doc/html/rfc9110)\n* Complete server support for HTTP/2 as defined in [RFC\n  9113](https://datatracker.ietf.org/doc/html/rfc9113) & [RFC\n  9110](https://datatracker.ietf.org/doc/html/rfc9110), comprehensively covered\n  by automated [h2spec](https://github.com/summerwind/h2spec) conformance\n  testing. Though deprecated by later RFCs, Bandit also supports h2c upgrades as\n  specified in [RFC7540](https://datatracker.ietf.org/doc/html/rfc7540)\n* Support for HTTP content encoding compression on both HTTP/1.x and HTTP/2.\n  gzip and deflate methods are supported per\n  [RFC9110§8.4.1.{2,3}](https://www.rfc-editor.org/rfc/rfc9110.html#section-8.4.1.2)\n* Complete server support for WebSockets as defined in [RFC\n  6455](https://datatracker.ietf.org/doc/html/rfc6455), comprehensively covered by automated\n  [Autobahn](https://github.com/crossbario/autobahn-testsuite) conformance testing. Per-message\n  compression as defined in [RFC 7692](https://datatracker.ietf.org/doc/html/rfc7692) is also\n  supported\n* Extremely scalable and performant client handling at a rate up to 4x that of Cowboy for the same\n  workload with as-good-or-better memory use\n\nAny Phoenix or Plug app should work with Bandit as a drop-in replacement for\nCowboy; exceptions to this are errors (if you find one, please [file an\nissue!](https://github.com/mtrudel/bandit/issues)).","ref":"readme.html#project-status","title":"Project Status - README","type":"extras"},{"doc":"Bandit fully supports Phoenix. Phoenix applications which use WebSockets for\nfeatures such as Channels or LiveView require Phoenix 1.7 or later.\n\nUsing Bandit to host your Phoenix application couldn't be simpler:\n\n1. Add Bandit as a dependency in your Phoenix application's `mix.exs`:\n\n    ```elixir\n    {:bandit, \"~> 1.0-pre\"}\n    ```\n2. Add the following `adapter:` line to your endpoint configuration in `config/config.exs`, as in the following example:\n\n     ```elixir\n     # config/config.exs\n\n     config :your_app, YourAppWeb.Endpoint,\n       adapter: Bandit.PhoenixAdapter, # <---- ADD THIS LINE\n       url: [host: \"localhost\"],\n       render_errors: ...\n     ```\n3. That's it! **You should now see messages at startup indicating that Phoenix is\n   using Bandit to serve your endpoint**, and everything should 'just work'. Note\n   that if you have set any exotic configuration options within your endpoint,\n   you may need to update that configuration to work with Bandit; see the\n   [Bandit.PhoenixAdapter](https://hexdocs.pm/bandit/Bandit.PhoenixAdapter.html)\n   documentation for more information.","ref":"readme.html#using-bandit-with-phoenix","title":"Using Bandit With Phoenix - README","type":"extras"},{"doc":"Using Bandit to host your own Plug is very straightforward. Assuming you have\na Plug module implemented already, you can host it within Bandit by adding\nsomething similar to the following to your application's `Application.start/2`\nfunction:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    {Bandit, plug: MyApp.MyPlug}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\nFor less formal usage, you can also start Bandit using the same configuration\noptions via the `Bandit.start_link/1` function:\n\n```elixir\n# Start an http server on the default port 4000, serving MyApp.MyPlug\nBandit.start_link(plug: MyPlug)\n```","ref":"readme.html#using-bandit-with-plug-applications","title":"Using Bandit With Plug Applications - README","type":"extras"},{"doc":"A number of options are defined when starting a server. The complete list is\ndefined by the [`t:Bandit.options/0`](https://hexdocs.pm/bandit/Bandit.html#summary) type.","ref":"readme.html#configuration","title":"Configuration - README","type":"extras"},{"doc":"By far the most common stumbling block encountered when setting up an HTTPS\nserver involves configuring key and certificate data. Bandit is comparatively\neasy to set up in this regard, with a working example looking similar to the\nfollowing:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    {Bandit,\n     plug: MyPlug,\n     scheme: :https,\n     certfile: \"/absolute/path/to/cert.pem\",\n     keyfile: \"/absolute/path/to/key.pem\"}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"readme.html#setting-up-an-https-server","title":"Setting up an HTTPS Server - README","type":"extras"},{"doc":"If you're using Bandit to run a Phoenix application as suggested above, there is\nnothing more for you to do; WebSocket support will 'just work'.\n\nIf you wish to interact with WebSockets at a more fundamental level, the\n[WebSock](https://hexdocs.pm/websock/WebSock.html) and\n[WebSockAdapter](https://hexdocs.pm/websock_adapter/WebSockAdapter.html) libraries\nprovides a generic abstraction for WebSockets (very similar to how Plug is\na generic abstraction on top of HTTP). Bandit fully supports all aspects of\nthese libraries.","ref":"readme.html#websocket-support","title":"WebSocket Support - README","type":"extras"},{"doc":"Bandit primarily consists of three protocol-specific implementations, one each\nfor [HTTP/1][], [HTTP/2][] and [WebSockets][]. Each of these implementations is\nlargely distinct from one another, and is described in its own README linked\nabove.\n\nIf you're just taking a casual look at Bandit or trying to understand how an\nHTTP server works, the [HTTP/1][] implementation is likely the best place to\nstart exploring.\n\n[HTTP/1]: lib/bandit/http1/README.md\n[HTTP/2]: lib/bandit/http2/README.md\n[WebSockets]: lib/bandit/websocket/README.md","ref":"readme.html#implementation-details","title":"Implementation Details - README","type":"extras"},{"doc":"Contributions to Bandit are very much welcome! Before undertaking any substantial work, please\nopen an issue on the project to discuss ideas and planned approaches so we can ensure we keep\nprogress moving in the same direction.\n\nAll contributors must agree and adhere to the project's [Code of\nConduct](https://github.com/mtrudel/bandit/blob/main/CODE_OF_CONDUCT.md).\n\nSecurity disclosures should be handled per Bandit's published [security policy](https://github.com/mtrudel/bandit/blob/main/SECURITY.md).","ref":"readme.html#contributing","title":"Contributing - README","type":"extras"},{"doc":"Bandit is [available in Hex](https://hex.pm/docs/publish). The package can be installed\nby adding `bandit` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bandit, \"~> 1.0-pre\"}\n  ]\nend\n```\n\nDocumentation can be found at [https://hexdocs.pm/bandit](https://hexdocs.pm/bandit).\n\n# License\n\nMIT","ref":"readme.html#installation","title":"Installation - README","type":"extras"},{"doc":"# HTTP/1 Handler\n\nIncluded in this folder is a complete `ThousandIsland.Handler` based implementation of HTTP/1.x as\ndefined in [RFC 9112](https://datatracker.ietf.org/doc/rfc9112).","ref":"HTTP1_README.md.html","title":"HTTP/1 Implementation Notes","type":"extras"},{"doc":"Within a Bandit server, an HTTP/1 connection is modeled as a single process.\nThis process is tied to the lifecycle of the underlying TCP connection; in the\ncase of an HTTP client which makes use of HTTP's keep-alive feature to make\nmultiple requests on the same connection, all of these requests will be serviced\nby this same process. \n\nThe execution model to handle a given request is quite straightforward: the\nunderlying [Thousand Island](https://github.com/mtrudel/thousand_island) library\nwill call `Bandit.HTTP1.Handler.handle_data/3`, which will then attmept to parse\nthe headers of the request by calling `Bandit.HTTP1.Adapter.read_headers/1`.\nAssuming the common case, this list of headers will then be passed to \n`Bandit.Pipeline.run/6`, which will construct a `Plug.Conn` structure to\nrepresent the request and subsequently pass it to the configured `Plug` module.\n\n# Testing\n\nAll of this is exhaustively tested. Tests are located in `request_test.exs`, and\nare broadly either concerned with testing network-facing aspects of the\nimplementation (ie: how well Bandit satisfies the relevant RFCs) or the Plug-facing \naspects of the implementation.\n\nUnfortunately, there is no HTTP/1 equivalent to the external h2spec test suite.","ref":"HTTP1_README.md.html#process-model","title":"Process model - HTTP/1 Implementation Notes","type":"extras"},{"doc":"# HTTP/2 Handler\n\nIncluded in this folder is a complete `ThousandIsland.Handler` based implementation of HTTP/2 as\ndefined in [RFC 9113](https://datatracker.ietf.org/doc/rfc9113).","ref":"HTTP2_README.md.html","title":"HTTP/2 Implementation Notes","type":"extras"},{"doc":"Within a Bandit server, an HTTP/2 connection is modeled as a set of processes:\n\n* 1 process per connection, a `Bandit.HTTP2.Handler` module implementing the\n  `ThousandIsland.Handler` behaviour, and;\n* 1 process per stream (i.e.: per HTTP request) within the connection, implemented as\n  a `Bandit.HTTP2.StreamTask` Task\n\nThe lifetimes of these processes correspond to their role; a connection process lives for as long\nas a client is connected, and a stream process lives only as long as is required to process\na single stream request within a connection. \n\nConnection processes are the 'root' of each connection's process group, and are supervised by\nThousand Island in the same manner that `ThousandIsland.Handler` processes are usually supervised\n(see the [project README](https://github.com/mtrudel/thousand_island) for details).\n\nStream processes are not supervised by design. The connection process starts new stream processes as required, and does so\nonce a complete header block for a new stream has been received. It starts stream processes via\na standard `start_link` call, and manages the termination of the resultant linked stream processes\nby handling `{:EXIT,...}` messages as described in the Elixir documentation. This approach is\naligned with the realities of the HTTP/2 model, insofar as if a connection process terminates\nthere is no reason to keep its constituent stream processes around, and if a stream process dies\nthe connection should be able to handle this without itself terminating. It also means that our\nprocess model is very lightweight - there is no extra supervision overhead present because no such\nsupervision is required for the system to function in the desired way.","ref":"HTTP2_README.md.html#process-model","title":"Process model - HTTP/2 Implementation Notes","type":"extras"},{"doc":"The overall structure of the implementation is managed by the `Bandit.HTTP2.Handler` module, and\nlooks like the following:\n\n1. Bytes are asynchronously received from ThousandIsland via the\n   `Bandit.HTTP2.Handler.handle_data/3` function\n2. Frames are parsed from these bytes by calling the `Bandit.HTTP2.Frame.deserialize/2`\n   function. If successful, the parsed frame(s) are returned. We retain any unparsed bytes in\n   a buffer in order to attempt parsing them upon receipt of subsequent data from the client\n3. Parsed frames are passed into the `Bandit.HTTP2.Connection` module along with a struct of \n   same module. Frames are applied against this struct in a vaguely FSM-like manner, using pattern \n   matching within the `Bandit.HTTP2.Connection.handle_frame/3` function. Any side-effects of\n   received frames are applied in these functions, and an updated connection struct is returned to\n   represent the updated connection state. These side-effects can take the form of starting stream\n   tasks, conveying data to running stream tasks, responding to the client with various frames, or\n   any number of other actions\n4. This process is repeated every time we receive data from the client until the\n   `Bandit.HTTP2.Connection` module indicates that the connection should be closed, either\n   normally or due to error. Note that frame deserialization may end up returning a connection\n   error if the parsed frames fail specific criteria (generally, the frame parsing modules are\n   responsible for identifying errors as described in [section\n   6](https://datatracker.ietf.org/doc/html/rfc9113#section-6) of RFC 9113). In these cases, the\n   failure is passed through to the connection module for processing in order to coordinate an\n   orderly shutdown or client notification as appropriate","ref":"HTTP2_README.md.html#reading-client-data","title":"Reading client data - HTTP/2 Implementation Notes","type":"extras"},{"doc":"The details of a particular stream are contained within a `Bandit.HTTP2.Stream` struct\n(as well as a `Bandit.HTTP2.StreamTask` process in the case of active streams). The\n`Bandit.HTTP2.StreamCollection` module manages a collection of streams, allowing for the memory\nefficient management of complete & yet unborn streams alongside active ones.\n\nOnce a complete header block has been read, a `Bandit.HTTP2.StreamTask` is started to manage the\nactual calling of the configured `Plug` module for this server, using the `Bandit.HTTP2.Adapter`\nmodule as the implementation of the `Plug.Conn.Adapter` behaviour. This adapter uses a simple\n`receive` pattern to listen for messages sent to it from the connection process, a pattern chosen\nbecause it allows for easy provision of the blocking-style API required by the `Plug.Conn.Adapter`\nbehaviour. Functions in the `Bandit.HTTP2.Adapter` behaviour which write data to the client use\n`GenServer` calls to the `Bandit.HTTP2.Handler` module in order to pass data to the connection\nprocess.\n\n# Testing\n\nAll of this is exhaustively tested. Tests are broken up primarily into `protocol_test.exs`, which\nis concerned with aspects of the implementation relating to protocol conformance and\nclient-facing concerns, while `plug_test.exs` is concerned with aspects of the implementation\nhaving to do with the Plug API and application-facing concerns. There are also more\nunit-style tests covering frame serialization and deserialization.\n\nIn addition, the `h2spec` conformance suite is run via a `System` wrapper & executes the entirety\nof the suite (in strict mode) against a running Bandit server.","ref":"HTTP2_README.md.html#processing-requests","title":"Processing requests - HTTP/2 Implementation Notes","type":"extras"},{"doc":"Some limitations and assumptions of this implementation:\n\n* This handler assumes that the HTTP/2 connection preface has already been consumed from the\n  client. The `Bandit.InitialHandler` module uses this preface to discriminate between various\n  HTTP versions when determining which handler to use\n* Priority frames are parsed and validated, but do not induce any action on the part of the\n  server. There is no priority assigned to respective streams in terms of processing; all streams\n  are run in parallel as soon as they arrive\n* While flow control is completely implemented here, the specific values used for upload flow\n  control (that is, the end that we control) are fixed. Specifically, we attempt to maintain\n  fairly large windows in order to not restrict client uploads (we 'slow-start' window changes\n  upon receipt of first byte, mostly to retain parity between connection and stream window\n  management since connection windows cannot be changed via settings). The majority of flow\n  control logic has been encapsulated in the `Bandit.HTTP2.FlowControl` module should future\n  refinement be required","ref":"HTTP2_README.md.html#limitations-and-assumptions","title":"Limitations and Assumptions - HTTP/2 Implementation Notes","type":"extras"},{"doc":"# WebSocket Handler\n\nIncluded in this folder is a complete `ThousandIsland.Handler` based implementation of WebSockets\nas defined in [RFC 6455](https://datatracker.ietf.org/doc/rfc6455).","ref":"WebSocket_README.md.html","title":"WebSocket Implementation Notes","type":"extras"},{"doc":"A good overview of this process is contained in this [ElixirConf EU\ntalk](https://www.youtube.com/watch?v=usKLrYl4zlY).\n\nUpgrading an HTTP connection to a WebSocket connection is coordinated by code\ncontained within several libraries, including Bandit,\n[WebSockAdapter](https://github.com/phoenixframework/websock_adapter), and\n[Plug](https://github.com/elixir-plug/plug). \n\nThe HTTP request containing the upgrade request is first passed to the user's\napplication as a standard Plug call. After inspecting the request and deeming it\na suitable upgrade candidate (via whatever policy the application dictates), the\nuser indicates a desire to upgrade the connection to a WebSocket by calling\n`Plug.Conn.upgrade_adapter/3` (this is most commonly done by calling\n`WebSockAdapter.upgrade/4`, which wraps this underlying call in\na server-agnostic manner). At the conclusion of the `Plug.call/2` callback, \n`Bandit.Pipeline` will then attempy to upgrade the underlying connection. As\npart of this upgrade process, `Bandit.DelegatingHandler` will switch the\nHandler for the connection to be `Bandit.WebSocket.Handler`. This will cause any\nfuture communication after the upgrade process to be handled directly by\nBandit's WebSocket stack.","ref":"WebSocket_README.md.html#upgrade-mechanism","title":"Upgrade mechanism - WebSocket Implementation Notes","type":"extras"},{"doc":"Within a Bandit server, a WebSocket connection is modeled as a single process.\nThis process is directly tied to the lifecycle of the underlying WebSocket\nconnection; when upgrading from HTTP/1, the existing HTTP/1 handler process\n'magically' becomes a WebSocket process by changing which Handler the\n`Bandit.DelegatingHandler` delegates to. \n\nThe execution model to handle a given request is quite straightforward: at\nupgrade time, the `Bandit.DelegatingHandler` will call `handle_connection/2` to\nallow the WebSocket handler to initialize any startup state. Connection state is\nmodeled by the `Bandit.WebSocket.Connection` struct and module.\n\nAll data subsequently received by the underlying [Thousand\nIsland](https://github.com/mtrudel/thousand_island) library will result in\na call to `Bandit.WebSocket.Handler.handle_data/3`, which will then attmept to\nparse the data into one or more WebSocket frames. Once a frame has been\nconstructed, it is them passed through to the configured `WebSock` handler by\nway of the underlying `Bandit.WebSocket.Connection`.\n\n# Testing\n\nAll of this is exhaustively tested. Tests are broken up primarily into `protocol_test.exs`, which\nis concerned with aspects of the implementation relating to protocol conformance and\nclient-facing concerns, while `sock_test.exs` is concerned with aspects of the implementation\nhaving to do with the WebSock API and application-facing concerns. There are also more\nunit-style tests covering frame serialization and deserialization.\n\nIn addition, the `autobahn` conformance suite is run via a `System` wrapper & executes the entirety\nof the suite against a running Bandit server.","ref":"WebSocket_README.md.html#process-model","title":"Process model - WebSocket Implementation Notes","type":"extras"}]}