searchNodes=[{"doc":"Bandit is an HTTP server for Plug and WebSock apps. As an HTTP server, Bandit's primary goal is to act as 'glue' between client connections managed by Thousand Island and application code defined via the Plug and/or WebSock APIs. As such there really isn't a whole lot of user-visible surface area to Bandit, and as a consequence the API documentation presented here is somewhat sparse. This is by design! Bandit is intended to 'just work' in almost all cases; the only thought users typically have to put into Bandit comes in the choice of which options (if any) they would like to change when starting a Bandit server. The sparseness of the Bandit API should not be taken as an indicator of the comprehensiveness or robustness of the project. Using Bandit With Phoenix Bandit fully supports Phoenix. Phoenix applications which use WebSockets for features such as Channels or LiveView require Phoenix 1.7 or later. Using Bandit to host your Phoenix application couldn't be simpler: Add Bandit as a dependency in your Phoenix application's mix.exs : { :bandit , &quot;~&gt; 1.0-pre&quot; } Add the following adapter: line to your endpoint configuration in config/config.exs , as in the following example: # config/config.exs config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter , # &lt;---- ADD THIS LINE url : [ host : &quot;localhost&quot; ] , render_errors : ... That's it! You should now see messages at startup indicating that Phoenix is using Bandit to serve your endpoint , and everything should 'just work'. Note that if you have set any exotic configuration options within your endpoint, you may need to update that configuration to work with Bandit; see the Bandit.PhoenixAdapter documentation for more information. Using Bandit With Plug Applications Using Bandit to host your own Plug is very straightforward. Assuming you have a Plug module implemented already, you can host it within Bandit by adding something similar to the following to your application's Application.start/2 function: def start ( _type , _args ) do children = [ { Bandit , plug : MyApp.MyPlug } ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end For less formal usage, you can also start Bandit using the same configuration options via the Bandit.start_link/1 function: # Start an http server on the default port 4000, serving MyApp.MyPlug Bandit . start_link ( plug : MyPlug ) Configuration A number of options are defined when starting a server. The complete list is defined by the t:Bandit.options/0 type. Setting up an HTTPS Server By far the most common stumbling block encountered when setting up an HTTPS server involves configuring key and certificate data. Bandit is comparatively easy to set up in this regard, with a working example looking similar to the following: def start ( _type , _args ) do children = [ { Bandit , plug : MyPlug , scheme : :https , certfile : &quot;/absolute/path/to/cert.pem&quot; , keyfile : &quot;/absolute/path/to/key.pem&quot; ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end WebSocket Support If you're using Bandit to run a Phoenix application as suggested above, there is nothing more for you to do; WebSocket support will 'just work'. If you wish to interact with WebSockets at a more fundamental level, the WebSock and WebSockAdapter libraries provides a generic abstraction for WebSockets (very similar to how Plug is a generic abstraction on top of HTTP). Bandit fully supports all aspects of these libraries.","ref":"Bandit.html","title":"Bandit","type":"module"},{"doc":"Starts a Bandit server using the provided arguments. See options/0 for specific options to pass to this function.","ref":"Bandit.html#start_link/1","title":"Bandit.start_link/1","type":"function"},{"doc":"Options to configure the deflate library used for HTTP compression","ref":"Bandit.html#t:deflate_options/0","title":"Bandit.deflate_options/0","type":"type"},{"doc":"Options to configure the HTTP/1 stack in Bandit enabled : Whether or not to serve HTTP/1 requests. Defaults to true max_request_line_length : The maximum permitted length of the request line (expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes max_header_length : The maximum permitted length of any single header (combined key &amp; value, expressed as the number of bytes on the wire) in an HTTP/1.1 request. Defaults to 10_000 bytes max_header_count : The maximum permitted number of headers in an HTTP/1.1 request. Defaults to 50 headers max_requests : The maximum number of requests to serve in a single HTTP/1.1 connection before closing the connection. Defaults to 0 (no limit) compress : Whether or not to attempt compression of responses via content-encoding negotiation as described in RFC9110§8.4 . Defaults to true deflate_options : A keyword list of options to set on the deflate library. A complete list can be found at deflate_options/0","ref":"Bandit.html#t:http_1_options/0","title":"Bandit.http_1_options/0","type":"type"},{"doc":"Options to configure the HTTP/2 stack in Bandit enabled : Whether or not to serve HTTP/2 requests. Defaults to true max_header_key_length : The maximum permitted length of any single header key (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes max_header_value_length : The maximum permitted length of any single header value (expressed as the number of decompressed bytes) in an HTTP/2 request. Defaults to 10_000 bytes max_header_count : The maximum permitted number of headers in an HTTP/2 request. Defaults to 50 headers max_requests : The maximum number of requests to serve in a single HTTP/2 connection before closing the connection. Defaults to 0 (no limit) default_local_settings : Options to override the default values for local HTTP/2 settings. Values provided here will override the defaults specified in RFC9113§6.5.2 compress : Whether or not to attempt compression of responses via content-encoding negotiation as described in RFC9110§8.4 . Defaults to true deflate_options : A keyword list of options to set on the deflate library. A complete list can be found at deflate_options/0","ref":"Bandit.html#t:http_2_options/0","title":"Bandit.http_2_options/0","type":"type"},{"doc":"Possible top-level options to configure a Bandit server plug : The Plug to use to handle connections. Can be specified as MyPlug or {MyPlug, plug_opts} scheme : One of :http or :https . If :https is specified, you will also need to specify valid certfile and keyfile values (or an equivalent value within thousand_island_options.transport_options ). Defaults to :http port : The TCP port to listen on. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options . If ia string value is passed, it will be parsed as an integer. Defaults to 4000 if scheme is :http , and 4040 if scheme is :https ip : The interface(s) to listen on. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options.transport_options . Can be specified as: {1, 2, 3, 4} for IPv4 addresses {1, 2, 3, 4, 5, 6, 7, 8} for IPv6 addresses :loopback for local loopback (ie: 127.0.0.1 ) :any for all interfaces (ie: 0.0.0.0 ) {:local, &quot;/path/to/socket&quot;} for a Unix domain socket. If this option is used, the port option must be set to 0 keyfile : The path to a file containing the SSL key to use for this server. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options.transport_options . If a relative path is used here, you will also need to set the otp_app parameter and ensure that the named file is part of your application build certfile : The path to a file containing the SSL certificate to use for this server. This option is offered as a convenience and actually sets the option of the same name within thousand_island_options.transport_options . If a relative path is used here, you will also need to set the otp_app parameter and ensure that the named file is part of your application build otp_app : Provided as a convenience when using relative paths for keyfile and certfile cipher_suite : Used to define a pre-selected set of ciphers, as described by Plug.SSL.configure/1 . Optional, can be either :strong or :compatible display_plug : The plug to use when describing the connection in logs. Useful for situations such as Phoenix code reloading where you have a 'wrapper' plug but wish to refer to the connection by the endpoint name startup_log : The log level at which Bandit should log startup info. Defaults to :info log level, can be set to false to disable it thousand_island_options : A list of options to pass to Thousand Island. Bandit sets some default values in this list based on your top-level configuration; these values will be overridden by values appearing here. A complete list can be found at ThousandIsland.options/0 http_1_options : A list of options to configure Bandit's HTTP/1 stack. A complete list can be found at http_1_options/0 http_2_options : A list of options to configure Bandit's HTTP/2 stack. A complete list can be found at http_2_options/0 websocket_options : A list of options to configure Bandit's WebSocket stack. A complete list can be found at websocket_options/0","ref":"Bandit.html#t:options/0","title":"Bandit.options/0","type":"type"},{"doc":"Options to configure the WebSocket stack in Bandit enabled : Whether or not to serve WebSocket upgrade requests. Defaults to true max_frame_size : The maximum size of a single WebSocket frame (expressed as a number of bytes on the wire). Defaults to 0 (no limit) validate_text_frames : Whether or not to validate text frames as being UTF-8. Strictly speaking this is required per RFC6455§5.6, however it can be an expensive operation and one that may be safely skipped in some situations. Defaults to true compress : Whether or not to allow per-message deflate compression globally. Note that upgrade requests still need to set the compress: true option in connection_opts on a per-upgrade basis for compression to be negotiated (see 'WebSocket Support' section below for details). Defaults to true","ref":"Bandit.html#t:websocket_options/0","title":"Bandit.websocket_options/0","type":"type"},{"doc":"Settings as defined in RFC9113§6.5.2","ref":"Bandit.HTTP2.Settings.html","title":"Bandit.HTTP2.Settings","type":"module"},{"doc":"A collection of settings as defined in RFC9113§6.5","ref":"Bandit.HTTP2.Settings.html#t:t/0","title":"Bandit.HTTP2.Settings.t/0","type":"type"},{"doc":"Logging conveniences for Bandit servers Allows dynamically adding and altering the log level used to trace connections within a Bandit server via the use of telemetry hooks. Should you wish to do your own logging or tracking of these events, a complete list of the telemetry events emitted by Bandit is described in the module documentation for Bandit.Telemetry . The logging included in this module is concerned specifically with protocol level events. Should you wish to log lower level transport concens, there are similar functions to these in the ThousandIsland.Logger module. Corresponding telemetry events are described in the module documentation for ThousandIsland.Telemetry .","ref":"Bandit.Logger.html","title":"Bandit.Logger","type":"module"},{"doc":"Start logging Bandit at the specified log level. Valid values for log level are :error and :info . Enabling a given log level implicitly enables all higher log levels as well.","ref":"Bandit.Logger.html#attach_logger/1","title":"Bandit.Logger.attach_logger/1","type":"function"},{"doc":"Stop logging Thousand Island at the specified log level. Disabling a given log level implicitly disables all lower log levels as well.","ref":"Bandit.Logger.html#detach_logger/1","title":"Bandit.Logger.detach_logger/1","type":"function"},{"doc":"Supported log levels","ref":"Bandit.Logger.html#t:log_level/0","title":"Bandit.Logger.log_level/0","type":"type"},{"doc":"A Bandit adapter for Phoenix. This adapter provides out-of-the-box support for all aspects of Phoenix 1.7 and later. Earlier versions of Phoenix will work with this adapter, but without support for WebSockets. To use this adapter, your project will need to include Bandit as a dependency: { :bandit , &quot;~&gt; 1.0-pre&quot; } Once Bandit is included as a dependency of your Phoenix project, add the following adapter: line to your endpoint configuration in config/config.exs , as in the following example: # config/config.exs config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter , # &lt;---- ADD THIS LINE url : [ host : &quot;localhost&quot; ] , render_errors : ... That's it! After restarting Phoenix you should see the startup message indicate that it is being served by Bandit , and everything should 'just work'. Note that if you have set any exotic configuration options within your endpoint, you may need to update that configuration to work with Bandit; see below for details. Endpoint configuration This adapter supports the standard Phoenix structure for endpoint configuration. Top-level keys for :http and :https are supported, and configuration values within each of those are interpreted as raw Bandit configuration as specified by Bandit.options/0 . Bandit's confguration supports all values used in a standard out-of-the-box Phoenix application, so if you haven't made any substantial changes to your endpoint configuration things should 'just work' for you. In the event that you have made advanced changes to your endpoint configuration, you may need to update this config to work with Bandit. Consult Bandit's documentation at Bandit.options/0 for details.","ref":"Bandit.PhoenixAdapter.html","title":"Bandit.PhoenixAdapter","type":"module"},{"doc":"The following telemetry spans are emitted by bandit [:bandit, :request, *] Represents Bandit handling a specific client HTTP request This span is started by the following event: [:bandit, :request, :start] Represents the start of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request This span is ended by the following event: [:bandit, :request, :stop] Represents the end of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units duration : The span duration, in :native units conn : The Plug.Conn representing this connection req_header_end_time : The time that header reading completed, in :native units req_body_start_time : The time that request body reading started, in :native units. req_body_end_time : The time that request body reading completed, in :native units req_line_bytes : The length of the request line, in octets. Includes all line breaks. Not included for HTTP/2 requests req_header_bytes : The length of the request headers, in octets. Includes all line breaks. Not included for HTTP/2 requests req_body_bytes : The length of the request body, in octets resp_start_time : The time that the response started, in :native units resp_end_time : The time that the response completed, in :native units. Not included for chunked responses resp_line_bytes : The length of the reponse line, in octets. Includes all line breaks. Not included for HTTP/2 requests resp_header_bytes : The length of the reponse headers, in octets. Includes all line breaks. Not included for HTTP/2 requests resp_body_bytes : The length of the reponse body, in octets. If the response is compressed, this is the size of the compressed payload as sent on the wire. Set to 0 for chunked responses resp_uncompressed_body_bytes : The length of the original, uncompressed body. Only included for responses which are compressed resp_compression_method : The method of compression, as sent in the Content-Encoding header of the response. Only included for responses which are compressed This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request error : The error that caused the span to end, if it ended in error The following events may be emitted within this span: [:bandit, :request, :exception] The request for this span ended unexpectedly This event contains the following measurements: monotonic_time : The time of this event, in :native units This event contains the following metadata: telemetry_span_context : A unique identifier for this span connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request kind : The kind of unexpected condition, typically :exit exception : The exception which caused this unexpected termination stacktrace : The stacktrace of the location which caused this unexpected termination [:bandit, :websocket, *] Represents Bandit handling a WebSocket connection This span is started by the following event: [:bandit, :websocket, :start] Represents the start of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units compress : Details about the compression configuration for this connection This event contains the following metadata: telemetry_span_context : A unique identifier for this span origin_telemetry_span_context : The span context of the Bandit :request span from which this connection originated connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request This span is ended by the following event: [:bandit, :websocket, :stop] Represents the end of the span This event contains the following measurements: monotonic_time : The time of this event, in :native units duration : The span duration, in :native units recv_text_frame_count : The number of text frames received recv_text_frame_bytes : The total number of bytes received in the payload of text frames recv_binary_frame_count : The number of binary frames received recv_binary_frame_bytes : The total number of bytes received in the payload of binary frames recv_ping_frame_count : The number of ping frames received recv_ping_frame_bytes : The total number of bytes received in the payload of ping frames recv_pong_frame_count : The number of pong frames received recv_pong_frame_bytes : The total number of bytes received in the payload of pong frames recv_connection_close_frame_count : The number of connection close frames received recv_connection_close_frame_bytes : The total number of bytes received in the payload of connection close frames recv_continuation_frame_count : The number of continuation frames received recv_continuation_frame_bytes : The total number of bytes received in the payload of continuation frames send_text_frame_count : The number of text frames sent send_text_frame_bytes : The total number of bytes sent in the payload of text frames send_binary_frame_count : The number of binary frames sent send_binary_frame_bytes : The total number of bytes sent in the payload of binary frames send_ping_frame_count : The number of ping frames sent send_ping_frame_bytes : The total number of bytes sent in the payload of ping frames send_pong_frame_count : The number of pong frames sent send_pong_frame_bytes : The total number of bytes sent in the payload of pong frames send_connection_close_frame_count : The number of connection close frames sent send_connection_close_frame_bytes : The total number of bytes sent in the payload of connection close frames send_continuation_frame_count : The number of continuation frames sent send_continuation_frame_bytes : The total number of bytes sent in the payload of continuation frames This event contains the following metadata: telemetry_span_context : A unique identifier for this span origin_telemetry_span_context : The span context of the Bandit :request span from which this connection originated connection_telemetry_span_context : The span context of the Thousand Island :connection span which contains this request error : The error that caused the span to end, if it ended in error","ref":"Bandit.Telemetry.html","title":"Bandit.Telemetry","type":"module"},{"doc":"See System.monotonic_time/0 .","ref":"Bandit.Telemetry.html#monotonic_time/0","title":"Bandit.Telemetry.monotonic_time/0","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#span_exception/4","title":"Bandit.Telemetry.span_exception/4","type":"function"},{"doc":"","ref":"Bandit.Telemetry.html#t:t/0","title":"Bandit.Telemetry.t/0","type":"opaque"},{"doc":"Raised by Bandit adapters if a body is attempted to be read more than once per request","ref":"Bandit.BodyAlreadyReadError.html","title":"Bandit.BodyAlreadyReadError","type":"exception"},{"doc":"","ref":"Bandit.HTTP2.Stream.StreamError.html","title":"Bandit.HTTP2.Stream.StreamError","type":"exception"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-10-28-jun-2023","title":"Changelog - 1.0.0-pre.10 (28 Jun 2023)","type":"extras"},{"doc":"Add support for Plug.Conn.inform/3 on HTTP/1 connections (#180) Add support for h2c upgrades (#186, thanks @alisinabh!) Internal refactoring of HTTP/1 content-length encoded body reads (#184, #190, thanks @asakura &amp; @moogle19!)","ref":"changelog.html#enhancements","title":"Changelog - Enhancements","type":"extras"},{"doc":"Bump Thousand Island to 1.0.0-pre.6 (gaining support for suspend/resume API) Drop Elixir 1.12 as a supported target (it should continue to work, but is no longer covered by CI)","ref":"changelog.html#changes","title":"Changelog - Changes","type":"extras"},{"doc":"Fix crash when Plug used Plug.Conn.get_peer_data/1 function on HTTP/1 connections (#170, thanks @moogle19!) Fix port behaviour when connecting over unix socket (#176, thanks @asakura &amp; @ibarchenkov!)","ref":"changelog.html#fixes","title":"Changelog - Fixes","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-9-16-jun-2023","title":"Changelog - 1.0.0-pre.9 (16 Jun 2023)","type":"extras"},{"doc":"Use new ThousandIsland APIs for socket info (#167, thanks @asakura!)","ref":"changelog.html#changes","title":"Changelog - Changes","type":"extras"},{"doc":"Handle nil connection close reason when closing a WebSocket","ref":"changelog.html#fixes","title":"Changelog - Fixes","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-8-15-jun-2023","title":"Changelog - 1.0.0-pre.8 (15 Jun 2023)","type":"extras"},{"doc":"Further improve logging on WebSocket upgrade errors (#149)","ref":"changelog.html#fixes","title":"Changelog - Fixes","type":"extras"},{"doc":"","ref":"changelog.html#1-0-0-pre-7-14-jun-2023","title":"Changelog - 1.0.0-pre.7 (14 Jun 2023)","type":"extras"},{"doc":"Refactor HTTP/1 read routines (#158 &amp; #166, thanks @asakura!) Improve logging on WebSocket upgrade errors (#149)","ref":"changelog.html#enhancements","title":"Changelog - Enhancements","type":"extras"},{"doc":"Override any content-length headers that may have been set by Plug (#165) Send content-length on HTTP/2 responses where appropriate (#165)","ref":"changelog.html#changes","title":"Changelog - Changes","type":"extras"},{"doc":"Send correct content-length header when sending deflated response (#151) Do not attempt to deflate if Plug sends a content-encoding header (#165) Improve corner case handling of content-length request header (#163, thanks @ryanwinchester!) Handle case where ThousandIsland returns error tuples on some helper routines (#162)","ref":"changelog.html#fixes","title":"Changelog - Fixes","type":"extras"},{"doc":"Changes Always use the declaed scheme if declared in a request-line or :scheme pseudo-header (#159) Internal tidying (thanks @asakura!)","ref":"changelog.html#1-0-0-pre-6-8-jun-2023","title":"Changelog - 1.0.0-pre.6 (8 Jun 2023)","type":"extras"},{"doc":"Enhancements Total overhaul of typespecs throughout the library (thanks @asakura!)","ref":"changelog.html#1-0-0-pre-5-2-jun-2023","title":"Changelog - 1.0.0-pre.5 (2 Jun 2023)","type":"extras"},{"doc":"Enhancements Performance / correctness improvements to header length validation (#143, thanks @moogle19!) Performance improvements to host header port parsing (#145 &amp; #147, thanks @ryanwinchester!) Improve WebSocket upgrade failure error messages to aid in diagnosis (#152) Changes Consolidate credo config (#146, thanks @ryanwinchester!) Fixes Fix error in suggested version dependencies during 1.0-pre series (#142, thanks @cvkmohan!)","ref":"changelog.html#1-0-0-pre-4-23-may-2023","title":"Changelog - 1.0.0-pre.4 (23 May 2023)","type":"extras"},{"doc":"Enhancements Respect read timeout for HTTP/1 keepalives (#140) Support Websock 0.5.1, including support for optional WebSock.terminate/2 (#131) Changes Use Req instead of Finch in tests (#137) Improve a few corner cases in tests (#136)","ref":"changelog.html#1-0-0-pre-3-3-may-2023","title":"Changelog - 1.0.0-pre.3 (3 May 2023)","type":"extras"},{"doc":"Fixes Don't require transport_options to be a keyword list (#130, thanks @justinludwig!)","ref":"changelog.html#1-0-0-pre-2-24-apr-2023","title":"Changelog - 1.0.0-pre.2 (24 Apr 2023)","type":"extras"},{"doc":"Changes Update Thousand Island dependency to 1.0-pre Changelog for 0.7.x","ref":"changelog.html#1-0-0-pre-1-21-apr-2023","title":"Changelog - 1.0.0-pre.1 (21 Apr 2023)","type":"extras"},{"doc":"Changes Bandit will now raise an error at startup if no plug is specified in config (thanks @moogle19!) Fixes Fix crash at startup when using otp_app option (thanks @moogle19!) Minor doc formatting fixes","ref":"changelog.html#0-7-7-11-apr-2023","title":"Changelog - 0.7.7 (11 Apr 2023)","type":"extras"},{"doc":"Changes BREAKING CHANGE Rename top-level options field to thousand_island_options BREAKING CHANGE Rename deflate_opts to deflate_options where used Massive overhaul of documentation to use types where possible Bandit now uses a term of the form {Bandit, ref()} for id in our child spec Bumped to Thousand Island 0.6.7. num_connections is now 16384 by default Enhancements Added top level support for the following convenience parameters: port can now be set at the top level of your configuration ip can now be set at the top level of your configuration keyfile and certfile can now be set at the top level of your configuration Transport options are now validated by Plug.SSL.configure/1 when starting an HTTPS server Rely on Thousand Island to validate options specified in thousand_island_options . This should avoid cases like #125 in the future.","ref":"changelog.html#0-7-6-9-apr-2023","title":"Changelog - 0.7.6 (9 Apr 2023)","type":"extras"},{"doc":"Changes Drop explicit support for Elixir 1.11 since we no longer test it in CI (should still work, just that it's now at-your-own-risk) Add logo to ex_doc and README Fixes Allow access to Thousand Island's underlying shutdown_timeout option Fix test errors that cropped up in OTP 26","ref":"changelog.html#0-7-5-4-apr-2023","title":"Changelog - 0.7.5 (4 Apr 2023)","type":"extras"},{"doc":"Changes Calling Plug.Conn adapter functions for HTTP/2 based requests are no longer restricted to being called from the process which called Plug.call/2 Enhancements Added startup_log to control whether / how Bandit logs the bound host &amp; port at startup (Thanks @danschultzer) Improved logging when the configured port is in use at startup (Thanks @danschultzer) Update to Thousand Island 0.6.5","ref":"changelog.html#0-7-4-27-mar-2023","title":"Changelog - 0.7.4 (27 Mar 2023)","type":"extras"},{"doc":"Enhancements Added advanced handler_module configuration option to options Fixes Support returning x-gzip as negotiated content-encoding (previously would negotiate a request for x-gzip as gzip )","ref":"changelog.html#0-7-3-20-mar-2023","title":"Changelog - 0.7.3 (20 Mar 2023)","type":"extras"},{"doc":"Enhancements Added HTTP compression via 'Content-Encoding' negotiation, enabled by default. Configuration is available; see Bandit docs for details Changes Minor refactor of internal HTTP/2 plumbing. No user visible changes","ref":"changelog.html#0-7-2-18-mar-2023","title":"Changelog - 0.7.2 (18 Mar 2023)","type":"extras"},{"doc":"Changes Update documentation &amp; messaging to refer to RFC911x RFCs where appropriate Validate top-level config options at startup Revise Phoenix adapter to support new config options Doc updates","ref":"changelog.html#0-7-1-17-mar-2023","title":"Changelog - 0.7.1 (17 Mar 2023)","type":"extras"},{"doc":"Enhancements Add configuration points for various parameters within the HTTP/1, HTTP/2 and WebSocket stacks. See Bandit docs for details Changelog for 0.6.x","ref":"changelog.html#0-7-0-17-mar-2023","title":"Changelog - 0.7.0 (17 Mar 2023)","type":"extras"},{"doc":"Changes Modified telemetry event payloads to match the conventions espoused by :telemetry.span/3 Default shutdown timeout is now 15s (up from 5s) Enhancements Update to Thosuand Island 0.6.4 (from 0.6.2)","ref":"changelog.html#0-6-11-17-mar-2023","title":"Changelog - 0.6.11 (17 Mar 2023)","type":"extras"},{"doc":"Enhancements Support explicit setting of WebSocket close codes &amp; reasons as added in WebSock 0.5.0","ref":"changelog.html#0-6-10-10-mar-2023","title":"Changelog - 0.6.10 (10 Mar 2023)","type":"extras"},{"doc":"Enhancements Add comprehensive Telemetry support within Bandit, as documented in the Bandit.Telemetry module Update our ThousandIsland dependnecy to pull in Thousand Island's newly updated Telemetry support as documented in the ThousandIsland.Telemetry module Fix parsing of host / request headers which contain IPv6 addresses (#97). Thanks @derekkraan! Changes Use Plug's list of response code reason phrases (#96). Thanks @jclem! Minor doc updates","ref":"changelog.html#0-6-9-20-feb-2023","title":"Changelog - 0.6.9 (20 Feb 2023)","type":"extras"},{"doc":"Changes Close WebSocket connections with a code of 1000 (instead of 1001) when shutting down the server (#89) Use 100 acceptor processes by default (instead of 10) Improvements to make WebSocket frame masking faster","ref":"changelog.html#0-6-8-31-jan-2023","title":"Changelog - 0.6.8 (31 Jan 2023)","type":"extras"},{"doc":"Enhancements Remove logging entirely when client connections do not contain a valid protocol Refactor WebSocket support for about a 20% performance lift Bug Fixes Add nodelay option to test suite to fix artificially slow WebSocket perf tests","ref":"changelog.html#0-6-7-17-jan-2023","title":"Changelog - 0.6.7 (17 Jan 2023)","type":"extras"},{"doc":"Enhancements Log useful message when a TLS connection is made to plaintext server (#74)","ref":"changelog.html#0-6-6-11-jan-2023","title":"Changelog - 0.6.6 (11 Jan 2023)","type":"extras"},{"doc":"Enhancements Update Thousand Island to 0.5.15 (quiets logging in timeout cases) Quiet logging in when client connections do not contain a valid protocol Refactor HTTP/1 for about a 20% performance lift Add WebSocket support to CI benchmark workflow Doc updates Bug Fixes Allow multiple instances of Bandit to be started in the same node (#75) Improve error handling in HTTP/1 when protocol errors are encountered (#74)","ref":"changelog.html#0-6-5-10-jan-2023","title":"Changelog - 0.6.5 (10 Jan 2023)","type":"extras"},{"doc":"Bandit is an HTTP server for Plug and WebSock apps. Bandit is written entirely in Elixir and is built atop Thousand Island . It can serve HTTP/1.x, HTTP/2 and WebSocket clients over both HTTP and HTTPS. It is written with correctness, clarity &amp; performance as fundamental goals. In ongoing automated performance tests , Bandit's HTTP/1.x engine is up to 4x faster than Cowboy depending on the number of concurrent requests. When comparing HTTP/2 performance, Bandit is up to 1.5x faster than Cowboy. This is possible because Bandit has been built from the ground up for use with Plug applications; this focus pays dividends in both performance and also in the approachability of the code base. Bandit also emphasizes correctness. Its HTTP/2 implementation scores 100% on the h2spec suite in strict mode, and its WebSocket implementation scores 100% on the Autobahn test suite, both of which run as part of Bandit's comprehensive CI suite. Extensive unit test, credo, dialyzer, and performance regression test coverage round out a test suite that ensures that Bandit is and will remain a platform you can count on. Lastly, Bandit exists to demystify the lower layers of infrastructure code. In a world where The New Thing is nearly always adding abstraction on top of abstraction, it's important to have foundational work that is approachable &amp; understandable by users above it in the stack.","ref":"readme.html","title":"README","type":"extras"},{"doc":"Implement comprehensive support for HTTP/1.0 through HTTP/2 &amp; WebSockets (and beyond) backed by obsessive RFC literacy and automated conformance testing Aim for minimal internal policy and HTTP-level configuration. Delegate to Plug &amp; WebSock as much as possible, and only interpret requests to the extent necessary to safely manage a connection &amp; fulfill the requirements of safely supporting protocol correctness Prioritize (in order): correctness, clarity, performance. Seek to remove the mystery of infrastructure code by being approachable and easy to understand Along with our companion library Thousand Island , become the go-to HTTP &amp; low-level networking stack of choice for the Elixir community by being reliable, efficient, and approachable","ref":"readme.html#project-goals","title":"README - Project Goals","type":"extras"},{"doc":"Complete support for Phoenix applications (WebSocket support requires Phoenix 1.7+) Complete support of the Plug API Complete support of the WebSock API Complete server support for HTTP/1.x as defined in RFC 9112 &amp; RFC 9110 Complete server support for HTTP/2 as defined in RFC 9113 &amp; RFC 9110 , comprehensively covered by automated h2spec conformance testing. Though deprecated by later RFCs, Bandit also supports h2c upgrades as specified in RFC7540 Support for HTTP content encoding compression on both HTTP/1.x and HTTP/2. gzip and deflate methods are supported per RFC9110§8.4.1.{2,3} Complete server support for WebSockets as defined in RFC 6455 , comprehensively covered by automated Autobahn conformance testing. Per-message compression as defined in RFC 7692 is also supported Extremely scalable and performant client handling at a rate up to 4x that of Cowboy for the same workload with as-good-or-better memory use Any Phoenix or Plug app should work with Bandit as a drop-in replacement for Cowboy; exceptions to this are errors (if you find one, please file an issue! ).","ref":"readme.html#project-status","title":"README - Project Status","type":"extras"},{"doc":"Bandit fully supports Phoenix. Phoenix applications which use WebSockets for features such as Channels or LiveView require Phoenix 1.7 or later. Using Bandit to host your Phoenix application couldn't be simpler: Add Bandit as a dependency in your Phoenix application's mix.exs : { :bandit , &quot;~&gt; 1.0-pre&quot; } Add the following adapter: line to your endpoint configuration in config/config.exs , as in the following example: # config/config.exs config :your_app , YourAppWeb.Endpoint , adapter : Bandit.PhoenixAdapter , # &lt;---- ADD THIS LINE url : [ host : &quot;localhost&quot; ] , render_errors : ... That's it! You should now see messages at startup indicating that Phoenix is using Bandit to serve your endpoint , and everything should 'just work'. Note that if you have set any exotic configuration options within your endpoint, you may need to update that configuration to work with Bandit; see the Bandit.PhoenixAdapter documentation for more information.","ref":"readme.html#using-bandit-with-phoenix","title":"README - Using Bandit With Phoenix","type":"extras"},{"doc":"Using Bandit to host your own Plug is very straightforward. Assuming you have a Plug module implemented already, you can host it within Bandit by adding something similar to the following to your application's Application.start/2 function: def start ( _type , _args ) do children = [ { Bandit , plug : MyApp.MyPlug } ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end For less formal usage, you can also start Bandit using the same configuration options via the Bandit.start_link/1 function: # Start an http server on the default port 4000, serving MyApp.MyPlug Bandit . start_link ( plug : MyPlug )","ref":"readme.html#using-bandit-with-plug-applications","title":"README - Using Bandit With Plug Applications","type":"extras"},{"doc":"A number of options are defined when starting a server. The complete list is defined by the t:Bandit.options/0 type.","ref":"readme.html#configuration","title":"README - Configuration","type":"extras"},{"doc":"By far the most common stumbling block encountered when setting up an HTTPS server involves configuring key and certificate data. Bandit is comparatively easy to set up in this regard, with a working example looking similar to the following: def start ( _type , _args ) do children = [ { Bandit , plug : MyPlug , scheme : :https , certfile : &quot;/absolute/path/to/cert.pem&quot; , keyfile : &quot;/absolute/path/to/key.pem&quot; ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end","ref":"readme.html#setting-up-an-https-server","title":"README - Setting up an HTTPS Server","type":"extras"},{"doc":"If you're using Bandit to run a Phoenix application as suggested above, there is nothing more for you to do; WebSocket support will 'just work'. If you wish to interact with WebSockets at a more fundamental level, the WebSock and WebSockAdapter libraries provides a generic abstraction for WebSockets (very similar to how Plug is a generic abstraction on top of HTTP). Bandit fully supports all aspects of these libraries.","ref":"readme.html#websocket-support","title":"README - WebSocket Support","type":"extras"},{"doc":"Bandit primarily consists of three protocol-specific implmenetations, one each for HTTP/1 , HTTP/2 and WebSockets . Each of these implementations is largely distinct from one another, and is described in its own README linked above. If you're just taking a casual look at Bandit or trying to understand how an HTTP server works, the HTTP/1 implementation is likely the best place to start exploring.","ref":"readme.html#implementation-details","title":"README - Implementation Details","type":"extras"},{"doc":"Contributions to Bandit are very much welcome! Before undertaking any substantial work, please open an issue on the project to discuss ideas and planned approaches so we can ensure we keep progress moving in the same direction. All contributors must agree and adhere to the project's Code of Conduct . Security disclosures should be handled per Bandit's published security policy .","ref":"readme.html#contributing","title":"README - Contributing","type":"extras"},{"doc":"Bandit is available in Hex . The package can be installed by adding bandit to your list of dependencies in mix.exs : def deps do [ { :bandit , &quot;~&gt; 1.0-pre&quot; } ] end Documentation can be found at https://hexdocs.pm/bandit . License MIT","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"Included in this folder is a complete ThousandIsland.Handler based implementation of HTTP/1.x as defined in RFC 9112 .","ref":"HTTP1_README.md.html","title":"HTTP/1 Implementation Notes","type":"extras"},{"doc":"Within a Bandit server, an HTTP/1 connection is modeled as a single process. This process is tied to the lifecycle of the underlying TCP connection; in the case of an HTTP client which makes use of HTTP's keep-alive feature to make multiple requests on the same connection, all of these requests will be serviced by this same process. The execution model to handle a given request is quite straightforward: the underlying Thousand Island library will call Bandit.HTTP1.Handler.handle_data/3 , which will then attmept to parse the headers of the request by calling Bandit.HTTP1.Adapter.read_headers/1 . Assuming the common case, this list of headers will then be passed to Bandit.Pipeline.run/6 , which will construct a Plug.Conn structure to represent the request and subsequently pass it to the configure Plug module. Testing All of this is exhaustively tested. Tests are located in request_test.exs , and are broadly either concerned with testing network-facing aspects of the implementation (ie: how well Bandit satisfies the relevant RFCs) or the Plug-facing aspects of the implementation. Unfortunately, there is no HTTP/1 equivalent to the external h2spec test suite.","ref":"HTTP1_README.md.html#process-model","title":"HTTP/1 Implementation Notes - Process model","type":"extras"},{"doc":"Included in this folder is a complete ThousandIsland.Handler based implementation of HTTP/2 as defined in RFC 9113 .","ref":"HTTP2_README.md.html","title":"HTTP/2 Implementation Notes","type":"extras"},{"doc":"Within a Bandit server, an HTTP/2 connection is modeled as a set of processes: 1 process per connection, a Bandit.HTTP2.Handler module implementing the ThousandIsland.Handler behaviour, and; 1 process per stream (i.e.: per HTTP request) within the connection, implemented as a Bandit.HTTP2.StreamTask Task The lifetimes of these processes correspond to their role; a connection process lives for as long as a client is connected, and a stream process lives only as long as is required to process a single stream request within a connection. Connection processes are the 'root' of each connection's process group, and are supervised by Thousand Island in the same manner that ThousandIsland.Handler processes are usually supervised (see the project README for details). Stream processes are not supervised by design. The connection process starts new stream processes as required, and does so once a complete header block for a new stream has been received. It starts stream processes via a standard start_link call, and manages the termination of the resultant linked stream processes by handling {:EXIT,...} messages as described in the Elixir documentation. This approach is aligned with the realities of the HTTP/2 model, insofar as if a connection process terminates there is no reason to keep its constituent stream processes around, and if a stream process dies the connection should be able to handle this without itself terminating. It also means that our process model is very lightweight - there is no extra supervision overhead present because no such supervision is required for the system to function in the desired way.","ref":"HTTP2_README.md.html#process-model","title":"HTTP/2 Implementation Notes - Process model","type":"extras"},{"doc":"The overall structure of the implementation is managed by the Bandit.HTTP2.Handler module, and looks like the following: Bytes are asynchronously received from ThousandIsland via the Bandit.HTTP2.Handler.handle_data/3 function Frames are parsed from these bytes by calling the Bandit.HTTP2.Frame.deserialize/2 function. If successful, the parsed frame(s) are returned. We retain any unparsed bytes in a buffer in order to attempt parsing them upon receipt of subsequent data from the client Parsed frames are passed into the Bandit.HTTP2.Connection module along with a struct of same module. Frames are applied against this struct in a vaguely FSM-like manner, using pattern matching within the Bandit.HTTP2.Connection.handle_frame/3 function. Any side-effects of received frames are applied in these functions, and an updated connection struct is returned to represent the updated connection state. These side-effects can take the form of starting stream tasks, conveying data to running stream tasks, responding to the client with various frames, or any number of other actions This process is repeated every time we receive data from the client until the Bandit.HTTP2.Connection module indicates that the connection should be closed, either normally or due to error. Note that frame deserialization may end up returning a connection error if the parsed frames fail specific criteria (generally, the frame parsing modules are responsible for identifying errors as described in section 6 of RFC 9113). In these cases, the failure is passed through to the connection module for processing in order to coordinate an orderly shutdown or client notification as appropriate","ref":"HTTP2_README.md.html#reading-client-data","title":"HTTP/2 Implementation Notes - Reading client data","type":"extras"},{"doc":"The details of a particular stream are contained within a Bandit.HTTP2.Stream struct (as well as a Bandit.HTTP2.StreamTask process in the case of active streams). The Bandit.HTTP2.StreamCollection module manages a collection of streams, allowing for the memory efficient management of complete &amp; yet unborn streams alongside active ones. Once a complete header block has been read, a Bandit.HTTP2.StreamTask is started to manage the actual calling of the configured Plug module for this server, using the Bandit.HTTP2.Adapter module as the implementation of the Plug.Conn.Adapter behaviour. This adapter uses a simple receive pattern to listen for messages sent to it from the connection process, a pattern chosen because it allows for easy provision of the blocking-style API required by the Plug.Conn.Adapter behaviour. Functions in the Bandit.HTTP2.Adapter behaviour which write data to the client use GenServer calls to the Bandit.HTTP2.Handler module in order to pass data to the connection process. Testing All of this is exhaustively tested. Tests are broken up primarily into protocol_test.exs , which is concerned with aspects of the implementation relating to protocol conformance and client-facing concerns, while plug_test.exs is concerned with aspects of the implementation having to do with the Plug API and application-facing concerns. There are also more unit-style tests covering frame serialization and deserialization. In addition, the h2spec conformance suite is run via a System wrapper &amp; executes the entirety of the suite (in strict mode) against a running Bandit server.","ref":"HTTP2_README.md.html#processing-requests","title":"HTTP/2 Implementation Notes - Processing requests","type":"extras"},{"doc":"Some limitations and assumptions of this implementation: This handler assumes that the HTTP/2 connection preface has already been consumed from the client. The Bandit.InitialHandler module uses this preface to discriminate between various HTTP versions when determining which handler to use Priority frames are parsed and validated, but do not induce any action on the part of the server. There is no priority assigned to respective streams in terms of processing; all streams are run in parallel as soon as they arrive While flow control is completely implemented here, the specific values used for upload flow control (that is, the end that we control) are fixed. Specifically, we attempt to maintain fairly large windows in order to not restrict client uploads (we 'slow-start' window changes upon receipt of first byte, mostly to retain parity between connection and stream window management since connection windows cannot be changed via settings). The majority of flow control logic has been encapsulated in the Bandit.HTTP2.FlowControl module should future refinement be required","ref":"HTTP2_README.md.html#limitations-and-assumptions","title":"HTTP/2 Implementation Notes - Limitations and Assumptions","type":"extras"},{"doc":"Included in this folder is a complete ThousandIsland.Handler based implementation of WebSockets as defined in RFC 6455 .","ref":"WebSocket_README.md.html","title":"WebSocket Implementation Notes","type":"extras"},{"doc":"Within a Bandit server, a WebSocket connection is modeled as a single process. This process is directly tied to the lifecycle of the underlying WebSocket connection; when upgrading from HTTP/1, the existing HTTP/1 handler process 'magically' becomes a WebSocket process by changing which Handler the Bandit.DelegatingHandler delegates to. The execution model to handle a given request is quite straightforward: at upgrade time, the Bandit.DelegatingHandler will call handle_connection/2 to allow the WebSocket handler to initialize any startup state. Connection state is modeled by the Bandit.WebSocket.Connection struct and module. All data subsequently received by the underlying Thousand Island library will result in a call to Bandit.WebSocket.Handler.handle_data/3 , which will then attmept to parse the data into one or more WebSocket frames. Once a frame has been constructed, it is them passed through to the configured WebSock handler by way of the underlying Bandit.WebSocket.Connection . Testing All of this is exhaustively tested. Tests are broken up primarily into protocol_test.exs , which is concerned with aspects of the implementation relating to protocol conformance and client-facing concerns, while sock_test.exs is concerned with aspects of the implementation having to do with the WebSock API and application-facing concerns. There are also more unit-style tests covering frame serialization and deserialization. In addition, the autobahn conformance suite is run via a System wrapper &amp; executes the entirety of the suite against a running Bandit server.","ref":"WebSocket_README.md.html#process-model","title":"WebSocket Implementation Notes - Process model","type":"extras"}]